#!/usr/bin/php
<?php                                       # -*- mode: PHP -*-
#
# Sample program for processing hardware reading
#   of an automatic rain gauge.
#
# Written by YIP Chi Lap [Beta] clyip@cs.hku.hk
# As demonstration code for Rain Gauge Design Competition
#
# Note that little code for error checking (e.g., cannot open file)
#   has been included; users are suggested to add in their own
#   error-checking code for robustness.
#
# 2006-04-01[6] Version 1.0
# * First release
#
######################################################################
###
### Constants and predefined values
###
######################################################################

#
# Map for calibration: calibrationMap[n] maps a measurement n
#   to a value that reflects the amount of rain.
#
# Example: 
#   If the hardware connected to a weight detector gives a reading of 34,
#   $calibrationMap[34] can be the amount of rain
#   that corresponds to the hardware reading of 34.
#
# Here, an identity map is used so that users of this 
# 
$calibrationMap=
 array(  0,   1,   2,   3,   4,   5,   6,   7, 
         8,   9,  10,  11,  12,  13,  14,  15,  
        16,  17,  18,  19,  20,  21,  22,  23,  
        24,  25,  26,  27,  28,  29,  30,  31,  
        32,  33,  34,  35,  36,  37,  38,  39,  
        40,  41,  42,  43,  44,  45,  46,  47,  
        48,  49,  50,  51,  52,  53,  54,  55,  
        56,  57,  58,  59,  60,  61,  62,  63,  
        64,  65,  66,  67,  68,  69,  70,  71,  
        72,  73,  74,  75,  76,  77,  78,  79,  
        80,  81,  82,  83,  84,  85,  86,  87,  
        88,  89,  90,  91,  92,  93,  94,  95,  
        96,  97,  98,  99, 100, 101, 102, 103, 
       104, 105, 106, 107, 108, 109, 110, 111, 
       112, 113, 114, 115, 116, 117, 118, 119, 
       120, 121, 122, 123, 124, 125, 126, 127, 
       128, 129, 130, 131, 132, 133, 134, 135, 
       136, 137, 138, 139, 140, 141, 142, 143, 
       144, 145, 146, 147, 148, 149, 150, 151, 
       152, 153, 154, 155, 156, 157, 158, 159, 
       160, 161, 162, 163, 164, 165, 166, 167, 
       168, 169, 170, 171, 172, 173, 174, 175, 
       176, 177, 178, 179, 180, 181, 182, 183, 
       184, 185, 186, 187, 188, 189, 190, 191, 
       192, 193, 194, 195, 196, 197, 198, 199, 
       200, 201, 202, 203, 204, 205, 206, 207, 
       208, 209, 210, 211, 212, 213, 214, 215, 
       216, 217, 218, 219, 220, 221, 222, 223, 
       224, 225, 226, 227, 228, 229, 230, 231, 
       232, 233, 234, 235, 236, 237, 238, 239, 
       240, 241, 242, 243, 244, 245, 246, 247, 
       248, 249, 250, 251, 252, 253, 254, 255);

# Take a sample every $samplingInterval second or longer
$minSamplingInterval=3;
$dataSourceFileName="COM4";
$dataStoreFileName="rainLog";

##########################################################################
###
### Real work here
###
##########################################################################
# Source of data: COM4 (A plain file, or for Windows, a serial port)
$datasrc=fopen($dataSourceFileName,"r");
# Data store: a file
$datastore=fopen($dataStoreFileName,"a");

# The first data item is unreliable, just read and discard it
$data=fscanf($datasrc,"%d");

$prevSamplingTime=time();
$prevValue=0;
# Real data sampling loop
while($currValueArray=fscanf($datasrc,"%d")) {
  $currValue=$currValueArray[0];
  $currSamplingTime=time();
  if ($currSamplingTime>=$prevSamplingTime+$minSamplingInterval) {
    $measureStartTime=date("Y-m-d G:i:s",$prevSamplingTime);
    $measureInterval=$currSamplingTime-$prevSamplingTime;
    $rainAmount=$calibrationMap[$currValue]-$calibrationMap[$prevValue];
    $prevValue=$currValue;

    echo "${measureStartTime},${measureInterval},${rainAmount}\n";
    fwrite($datastore,"${measureStartTime},${measureInterval},${rainAmount}\n");

     /*
      You can generate an HTTP POST request here
        with values associated with the following keys:
	      teamNumber
	      password
	      measureStartTime
	      measureInterval
	      rainAmount
     */

    # Since it may take time to write data to file
    #   or send data to website,
    #   we take a new time() value here to make sure that
    #   the time we start waiting for input is faithfully recorded.
    $prevSamplingTime=time();
  }
}

fclose($datasrc);
?>
